==> dmarc-metrics-exporter-0.2.2.tar.gz: dmarc-metrics-exporter-0.2.2/dmarc_metrics_exporter/tests/conftest.py <==
import asyncio
import smtplib
import time
from contextlib import contextmanager
from dataclasses import astuple, dataclass
from email.message import EmailMessage
from typing import Any, Awaitable, Callable, Generator, Union

import docker.models
import pytest

from dmarc_metrics_exporter.imap_queue import ConnectionConfig, ImapClient


@dataclass
class NetworkAddress:
    host: str
    port: int


@dataclass
class Greenmail:
    smtp: NetworkAddress
    imap: ConnectionConfig


@pytest.fixture(name="docker_client")
def fixture_docker_client() -> docker.DockerClient:
    return docker.from_env()


@pytest.fixture(name="greenmail")
def fixture_greenmail(
    docker_client: docker.DockerClient,
) -> Generator[Greenmail, None, None]:
    with run_greenmail(docker_client) as greenmail:
        yield greenmail


@contextmanager
def run_greenmail(
    docker_client: docker.DockerClient,
) -> Generator[Greenmail, None, None]:
    container = docker_client.containers.run(
        "greenmail/standalone:1.6.0",
        detach=True,
        remove=True,
        ports={"3025/tcp": 3025, "3993/tcp": 3993},
    )
    yield Greenmail(
        smtp=NetworkAddress("localhost", 3025),
        imap=ConnectionConfig(
            host="localhost", port=3993, username="queue@localhost", password="password"
        ),
    )
    container.stop()


async def try_until_success(
    function: Union[Callable[[], Awaitable], Callable[[], Any]],
    timeout_seconds: int = 10,
    max_fn_duration_seconds: int = 1,
    poll_interval_seconds: float = 0.1,
):
    timeout = time.time() + timeout_seconds
    last_err = None
    while time.time() < timeout:
        try:
            result = function()
            if hasattr(result, "__await__"):
                return await asyncio.wait_for(result, max_fn_duration_seconds)
            else:
                return result
        except Exception as err:  # pylint: disable=broad-except
            last_err = err
            await asyncio.sleep(poll_interval_seconds)
    raise TimeoutError(
        f"Call to {function} not successful within {timeout_seconds} seconds."
    ) from last_err


async def send_email(msg: EmailMessage, network_address: NetworkAddress):
    smtp = smtplib.SMTP(*astuple(network_address))
    smtp.send_message(msg)
    smtp.quit()


async def verify_email_delivered(connection: ConnectionConfig, mailboxes=("INBOX",)):
    async with ImapClient(connection) as client:
        msg_counts = await asyncio.gather(
            *(client.select(mailbox) for mailbox in mailboxes)
        )
        assert any(count > 0 for count in msg_counts)


async def verify_imap_available(connection: ConnectionConfig):
    async with ImapClient(connection):
        pass

==> dmarc-metrics-exporter-0.4.0.tar.gz: dmarc-metrics-exporter-0.4.0/dmarc_metrics_exporter/tests/conftest.py <==
import asyncio
import smtplib
import time
from dataclasses import astuple, dataclass
from email.message import EmailMessage
from typing import Any, Awaitable, Callable, Union

import pytest
import requests

from dmarc_metrics_exporter.imap_queue import ConnectionConfig, ImapClient


@dataclass
class NetworkAddress:
    host: str
    port: int


@dataclass
class Greenmail:
    smtp: NetworkAddress
    imap: ConnectionConfig
    api: NetworkAddress

    @property
    def api_url(self) -> str:
        return f"http://{self.api.host}:{self.api.port}/api"

    def is_ready(self) -> bool:
        return (
            requests.get(f"{self.api_url}/service/readiness").status_code
            == requests.codes.ok
        )

    def purge_mails(self):
        requests.post(f"{self.api_url}/mail/purge").raise_for_status()

    async def restart(self):
        requests.post(f"{self.api_url}/service/reset")
        await try_until_success(self.is_ready)


@pytest.fixture(name="greenmail")
def fixture_greenmail() -> Greenmail:
    greenmail = Greenmail(
        smtp=NetworkAddress("localhost", 3025),
        imap=ConnectionConfig(
            host="localhost", port=3993, username="queue@localhost", password="password"
        ),
        api=NetworkAddress("localhost", 8080),
    )
    greenmail.purge_mails()
    return greenmail


async def try_until_success(
    function: Union[Callable[[], Awaitable], Callable[[], Any]],
    timeout_seconds: int = 10,
    max_fn_duration_seconds: int = 1,
    poll_interval_seconds: float = 0.1,
):
    timeout = time.time() + timeout_seconds
    last_err = None
    while time.time() < timeout:
        try:
            result = function()
            if hasattr(result, "__await__"):
                return await asyncio.wait_for(result, max_fn_duration_seconds)
            else:
                return result
        except Exception as err:  # pylint: disable=broad-except
            last_err = err
            await asyncio.sleep(poll_interval_seconds)
    raise TimeoutError(
        f"Call to {function} not successful within {timeout_seconds} seconds."
    ) from last_err


async def send_email(msg: EmailMessage, network_address: NetworkAddress):
    smtp = smtplib.SMTP(*astuple(network_address))
    smtp.send_message(msg)
    smtp.quit()


async def verify_email_delivered(connection: ConnectionConfig, mailboxes=("INBOX",)):
    async with ImapClient(connection) as client:
        msg_counts = await asyncio.gather(
            *(client.select(mailbox) for mailbox in mailboxes)
        )
        assert any(count > 0 for count in msg_counts)
